package com.jjt.calc.controller;

import java.util.List;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.jjt.common.annotation.Log;
import com.jjt.common.core.controller.BaseController;
import com.jjt.common.core.domain.AjaxResult;
import com.jjt.common.enums.BusinessType;
import com.jjt.calc.domain.TwinCalcHourEnergy;
import com.jjt.calc.service.ITwinCalcHourEnergyService;
import com.jjt.common.utils.poi.ExcelUtil;
import com.jjt.common.core.page.TableDataInfo;

/**
 * 能源1小时统计数据Controller
 *
 * @author wukai
 * @date 2025-01-18
 */
@Api(tags="能源1小时统计数据")
@RestController
@RequestMapping("/calc/energy")
public class TwinCalcHourEnergyController extends BaseController{
    @Resource
    private ITwinCalcHourEnergyService twinCalcHourEnergyService;

    /**
     * 查询能源1小时统计数据列表
     */
    @ApiOperation("查询能源1小时统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:energy:list')")
    @GetMapping("/list")
    public TableDataInfo list(TwinCalcHourEnergy twinCalcHourEnergy)
    {
        startPage();
        List<TwinCalcHourEnergy> list = twinCalcHourEnergyService.selectTwinCalcHourEnergyList(twinCalcHourEnergy);
        return getDataTable(list);
    }

    /**
     * 导出能源1小时统计数据列表
     */
    @ApiOperation("导出能源1小时统计数据列表")
    @PreAuthorize("@ss.hasPermi('calc:energy:export')")
    @Log(title = "能源1小时统计数据", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TwinCalcHourEnergy twinCalcHourEnergy)
    {
        List<TwinCalcHourEnergy> list = twinCalcHourEnergyService.selectTwinCalcHourEnergyList(twinCalcHourEnergy);
        ExcelUtil<TwinCalcHourEnergy> util = new ExcelUtil<TwinCalcHourEnergy>(TwinCalcHourEnergy.class);
        util.exportExcel(response, list, "能源1小时统计数据数据");
    }

    /**
     * 获取能源1小时统计数据详细信息
     */
    @ApiOperation("获取能源1小时统计数据详细信息")
    @PreAuthorize("@ss.hasPermi('calc:energy:query')")
    @GetMapping(value = "/{autoId}")
    public AjaxResult getInfo(@PathVariable("autoId") Long autoId)
    {
        return success(twinCalcHourEnergyService.selectTwinCalcHourEnergyByAutoId(autoId));
    }

    /**
     * 新增能源1小时统计数据
     */
    @ApiOperation("新增能源1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:energy:add')")
    @Log(title = "能源1小时统计数据", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TwinCalcHourEnergy twinCalcHourEnergy)
    {
        return toAjax(twinCalcHourEnergyService.insertTwinCalcHourEnergy(twinCalcHourEnergy));
    }

    /**
     * 修改能源1小时统计数据
     */
    @ApiOperation("修改能源1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:energy:edit')")
    @Log(title = "能源1小时统计数据", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TwinCalcHourEnergy twinCalcHourEnergy)
    {
        return toAjax(twinCalcHourEnergyService.updateTwinCalcHourEnergy(twinCalcHourEnergy));
    }

    /**
     * 删除能源1小时统计数据
     */
    @ApiOperation("删除能源1小时统计数据")
    @PreAuthorize("@ss.hasPermi('calc:energy:remove')")
    @Log(title = "能源1小时统计数据", businessType = BusinessType.DELETE)
	@DeleteMapping("/{autoIds}")
    public AjaxResult remove(@PathVariable Long[] autoIds)
    {
        return toAjax(twinCalcHourEnergyService.deleteTwinCalcHourEnergyByAutoIds(autoIds));
    }
}
